@@ -61,6 +61,8 @@
 
 #if QT_CONFIG(evr)
 #include "directshowevrvideowindowcontrol.h"
+#else
+#include <mmreg.h>
 #endif
 
 #include "qmediacontent.h"
@@ -113,7 +115,7 @@ public:
     }
 
 protected:
-    void run() { m_service->run(); }
+    void run() override { m_service->run(); }
 
 private:
     DirectShowPlayerService *m_service;
@@ -214,14 +216,16 @@ QMediaControl *DirectShowPlayerService::requestControl(const char *name)
         }
     } else if (qstrcmp(name, QVideoWindowControl_iid) == 0) {
         if (!m_videoRendererControl && !m_videoWindowControl) {
-            IBaseFilter *filter;
+            IBaseFilter *filter{};
 
 #if QT_CONFIG(evr)
-            DirectShowEvrVideoWindowControl *evrControl = new DirectShowEvrVideoWindowControl;
-            if ((filter = evrControl->filter()))
-                m_videoWindowControl = evrControl;
-            else
-                delete evrControl;
+            if (!qgetenv("QT_DIRECTSHOW_NO_EVR").toInt()) {
+                DirectShowEvrVideoWindowControl *evrControl = new DirectShowEvrVideoWindowControl;
+                if ((filter = evrControl->filter()))
+                    m_videoWindowControl = evrControl;
+                else
+                    delete evrControl;
+            }
 #endif
             // Fall back to the VMR9 if the EVR is not available
             if (!m_videoWindowControl) {
@@ -399,8 +403,6 @@ void DirectShowPlayerService::doSetUrlSource(QMutexLocker *locker)
             m_pendingTasks |= SetRate;
 
         m_source = source;
-    } else if (!m_resources.isEmpty()) {
-        m_pendingTasks |= SetUrlSource;
     } else {
         m_graphStatus = InvalidMedia;
 
@@ -521,9 +523,9 @@ void DirectShowPlayerService::doRender(QMutexLocker *locker)
                         peer->Release();
                     } else {
                         locker->unlock();
-                        HRESULT hr;
-                        if (SUCCEEDED(hr = graph->RenderEx(
-                                pin, /*AM_RENDEREX_RENDERTOEXISTINGRENDERERS*/ 1, 0))) {
+                        HRESULT hr = graph->RenderEx(pin, /*AM_RENDEREX_RENDERTOEXISTINGRENDERERS*/ 1, 0);
+                        // Do not return an error if no video output is set yet.
+                        if (SUCCEEDED(hr) || !(m_executedTasks & SetVideoOutput)) {
                             rendered = true;
                         } else if (renderHr == S_OK || renderHr == VFW_E_NO_DECOMPRESSOR){
                             renderHr = hr;
@@ -947,7 +949,6 @@ void DirectShowPlayerService::pause()
 
     if (m_executedTasks & Render) {
         if (m_executedTasks & Stop) {
-            m_atEnd = false;
             if (m_seekPosition == -1) {
                 m_dontCacheNextSeekResult = true;
                 m_seekPosition = 0;
@@ -973,7 +974,8 @@ void DirectShowPlayerService::doPause(QMutexLocker *locker)
         control->Release();
 
         if (SUCCEEDED(hr)) {
-            if (IMediaSeeking *seeking = com_cast<IMediaSeeking>(m_graph, IID_IMediaSeeking)) {
+            IMediaSeeking *seeking = com_cast<IMediaSeeking>(m_graph, IID_IMediaSeeking);
+            if (!m_atEnd && seeking) {
                 LONGLONG position = 0;
 
                 seeking->GetCurrentPosition(&position);
@@ -982,6 +984,7 @@ void DirectShowPlayerService::doPause(QMutexLocker *locker)
                 m_position = position / qt_directShowTimeScale;
             } else {
                 m_position = 0;
+                m_atEnd = false;
             }
 
             m_executedTasks |= Pause;
@@ -1092,9 +1095,9 @@ qint64 DirectShowPlayerService::position() const
     QMutexLocker locker(const_cast<QMutex *>(&m_mutex));
 
     if (m_graphStatus == Loaded) {
-        if (m_executingTask == Seek || m_executingTask == SetRate || (m_pendingTasks & Seek)) {
+        if (m_executingTask == Seek || m_executingTask == SetRate || (m_pendingTasks & Seek))
             return m_position;
-        } else if (IMediaSeeking *seeking = com_cast<IMediaSeeking>(m_graph, IID_IMediaSeeking)) {
+        if (IMediaSeeking *seeking = com_cast<IMediaSeeking>(m_graph, IID_IMediaSeeking)) {
             LONGLONG position = 0;
 
             seeking->GetCurrentPosition(&position);
@@ -1113,9 +1116,9 @@ QMediaTimeRange DirectShowPlayerService::availablePlaybackRanges() const
     QMutexLocker locker(const_cast<QMutex *>(&m_mutex));
 
     if (m_graphStatus == Loaded) {
-        if (m_executingTask == Seek || m_executingTask == SetRate || (m_pendingTasks & Seek)) {
+        if (m_executingTask == Seek || m_executingTask == SetRate || (m_pendingTasks & Seek))
             return m_playbackRange;
-        } else if (IMediaSeeking *seeking = com_cast<IMediaSeeking>(m_graph, IID_IMediaSeeking)) {
+        if (IMediaSeeking *seeking = com_cast<IMediaSeeking>(m_graph, IID_IMediaSeeking)) {
             LONGLONG minimum = 0;
             LONGLONG maximum = 0;
 
@@ -1196,9 +1199,8 @@ int DirectShowPlayerService::bufferStatus() const
         reader->Release();
 
         return percentage;
-    } else {
-        return 0;
     }
+    return 0;
 #else
     return 0;
 #endif
@@ -1613,7 +1615,7 @@ void DirectShowPlayerService::updateStatus()
         m_playerControl->updateStatus(QMediaPlayer::LoadingMedia);
         break;
     case Loaded:
-        if ((m_pendingTasks | m_executingTask | m_executedTasks) & (Play | Pause)) {
+        if ((m_executingTask | m_executedTasks) & (Play | Pause)) {
             if (m_buffering)
                 m_playerControl->updateStatus(QMediaPlayer::BufferingMedia);
             else
